@@ -11,7 +11,12 @@ def camel_to_snake(name):
 
 class XeDoc(rframe.BaseSchema):
     _ALIAS: ClassVar = ""
+    _CATEGORY: ClassVar = "general"
+
     _XEDOCS = {}
+    
+    class Config:
+        allow_population_by_field_name = True
 
     def __init_subclass__(cls) -> None:
 
@@ -50,7 +55,7 @@ class XeDoc(rframe.BaseSchema):
 class VersionedXeDoc(XeDoc):
     _ALIAS = ""
     
-    version: str = rframe.Index()
+    version: str = rframe.Index(max_length=20)
 
     def pre_delete(self, datasource):
         raise IndexError('Versioned documents are append-only.')
