@@ -1,4 +1,6 @@
+import io
 import math
+import json
 import xedocs
 import rframe
 import datetime
@@ -45,15 +47,40 @@ from typing import Mapping, ClassVar, Type
 from concurrent.futures import ThreadPoolExecutor
 
 from .schemas import XeDoc
-
+from .utils import docs_to_wiki, docs_to_dataframe
 
 executor = ThreadPoolExecutor(max_workers=5)  # pylint: disable=consider-using-with
 logger = logging.getLogger(__name__)
 
 @json_serializable.dispatch(precedence=1)
-def json_serializable(value: rframe.Interval):
+def json_serializable(value: rframe.IntegerInterval):
     return value.left, value.right
 
+@json_serializable.dispatch(precedence=1)
+def json_serializable(value: rframe.TimeInterval):
+    return f"{str(value.left)} to {str(value.right)}"
+
+
+def make_csv(schema, docs):
+    df = docs_to_dataframe(schema, docs)
+    return io.StringIO(df.to_csv())
+
+
+def make_json(schema, docs):
+    docs= [doc.dict() for doc in docs]
+    return io.StringIO(json.dumps(docs))
+
+
+def make_wiki(schema, docs):
+    return io.StringIO(docs_to_wiki(schema, docs))
+
+
+download_options = {
+    'csv': make_csv,
+    'json': make_json,
+    'wiki': make_wiki,
+}
+
 
 class NullableInput(LiteralInput):
     def _process_property_change(self, msg):
@@ -367,7 +394,7 @@ class QueryEditor(CompositeWidget):
             if value not in options:
                 value = options[0]
             widget = pn.widgets.MultiChoice(name=alias + 's', 
-                                   value=[value] if value else [], 
+                                   value = [], 
                                    options=options)
         else:
             try:
@@ -446,7 +473,7 @@ class XeDocListEditor(CompositeWidget):
         self._composite[:] = [pn.panel(self.table_view)]
         
     @param.depends('class_', 'pagination', 'page_size', watch=True, on_init=True)
-    def _class_changed(self):
+    def _make_table_widget(self):
 
         if len(self.value) > self.MAX_SEND_ROWS:
             self.pagination = 'remote'
@@ -487,8 +514,7 @@ class XeDocListEditor(CompositeWidget):
         
     @param.depends('value', watch=True, )
     def _update_table(self):
-        if not self.value:
-            return
+        
 
         if len(self.value) > self.MAX_SEND_ROWS:
             self.pagination = 'remote'
@@ -499,6 +525,9 @@ class XeDocListEditor(CompositeWidget):
         if self.table_widget is None:
             return
 
+        if not len(self.table_widget.value) or not self.value:
+            return self._make_table_widget()
+
         assert all([isinstance(doc, self.class_) 
                     for doc in self.value]), f"Value must be a list of {self.class_} instances"
 
@@ -559,7 +588,7 @@ class ModelTableEditor(pn.viewable.Viewer):
     page = param.Integer(-1, bounds=(-1, 1))
     last_page = param.Integer(0)
     page_size = param.Integer(15)
-    
+
     refresh_table = param.Event()
     table = param.Parameter()
     
@@ -585,7 +614,7 @@ class ModelTableEditor(pn.viewable.Viewer):
                          ['class_', 'page', 'query', 'refresh_table'])
 
     @param.depends('class_', watch=True, on_init=True)
-    def _class_changed(self):
+    def _make_table_widget(self):
         docs = [json_serializable(doc.index_labels) for doc in self.docs]
         df = pd.DataFrame(docs, columns=list(self.class_.get_index_fields()))
 
@@ -607,6 +636,13 @@ class ModelTableEditor(pn.viewable.Viewer):
     
     @param.depends('docs', watch=True)
     def _docs_changed(self):
+        if self.table is None:
+            self._make_table_widget()
+            return
+
+        if not len(self.table.value):
+            return self._make_table_widget()
+
         docs = [json_serializable(doc.index_labels) for doc in self.docs]
         df = pd.DataFrame(docs, columns=list(self.class_.get_index_fields()))
         self.table.value = df
@@ -630,9 +666,10 @@ class ModelTableEditor(pn.viewable.Viewer):
             self.page = 0
         if self.page == -1:
             self.page = 0
-        end = math.ceil(self.class_.compile_query(**self.query).count()/self.page_size)
-        self.param.page.bounds = (0, end)
-        self.last_page = end
+        # end = math.ceil(self.class_.compile_query(**self.query).count()/self.page_size)
+        # self.param.page.bounds = (0, end)
+        # self.last_page = end
+        self._update_page_range()
         self._update_docs()
         
     def trigger_refresh_cb(self, event):
@@ -644,6 +681,13 @@ class ModelTableEditor(pn.viewable.Viewer):
     def decrement_page(self, event):
         self.page -= 1
 
+    def _set_page_range(self, future):
+        count = future.result()
+        last_page =  math.ceil(count/self.page_size)
+        with unlocked():
+            self.param.page.bounds = (0, last_page)
+            self.last_page = last_page
+
     def _set_docs(self, future):
         docs = future.result()
 
@@ -666,12 +710,26 @@ class ModelTableEditor(pn.viewable.Viewer):
             
         return docs
 
+    def _get_page_count(self):
+        count = 1
+        try:
+            count = self.class_.compile_query(**self.query).count()
+        except Exception as e:
+            logger.error(str(e))
+        return count
+
     def _update_docs(self, *events):
         self.table.loading = True
         loop = IOLoop.current()
         future = executor.submit(self._get_page)
         loop.add_future(future, self._set_docs)
 
+    def _update_page_range(self, *events):
+        self.table.loading = True
+        loop = IOLoop.current()
+        future = executor.submit(self._get_page_count)
+        loop.add_future(future, self._set_page_range)
+
     def value_editor(self, row):
         if len(self.docs)<=row.name:
             return pn.Column()
@@ -748,6 +806,35 @@ class ModelTableEditor(pn.viewable.Viewer):
         return pn.Column(self.model_editor,
                          width_policy='min')
 
+    pn.depends('class_')
+    def download_panel(self):
+
+        current_only = pn.widgets.RadioBoxGroup(options={'Current page': 1,
+                                                         'Entire query': 0}, 
+                                                inline=True)
+        filename = pn.widgets.TextInput(name='Filename', value=self.class_._ALIAS)
+
+        filetype = pn.widgets.Select(name='Format', options=list(download_options))
+        
+        def cb():
+            if current_only.value:
+                docs = self.docs
+            else:
+                docs = self.class_.find(**self.query)
+            return download_options[filetype.value](self.class_, docs)
+
+        download_button = pn.widgets.FileDownload(filename=f'{self.class_._ALIAS}.{filetype.value}',
+                                                  callback=cb)
+
+        def update_filename(*events):
+            download_button.filename = f'{filename.value}.{filetype.value}'
+
+        filename.param.watch(update_filename, 'value')
+        filetype.param.watch(update_filename, 'value')
+
+        return pn.Column(current_only, filename, filetype, download_button)
+
+
     def __panel__(self):
         right_panel = pn.Column(self.page_controls,
                                 self.table_panel)
@@ -857,6 +944,12 @@ class XedocsEditor(pn.viewable.Viewer):
                               self.editor.find_button, 
                               pn.layout.Spacer(width_policy='max'))
 
+    @pn.depends('editor')
+    def download_panel(self):
+        if self.editor is None:
+            return pn.Row(pn.layout.Spacer(width_policy='max'))
+        return pn.panel(self.editor.download_panel)
+
     def __panel__(self):
         bottom_panel = pn.Row(self.query_or_insert_panel, self.data_panel)
         return pn.Column(self.selection_panel,
